@@ -8,7 +8,7 @@ use warnings;
 use Carp;
 use List::Util qw{ max min };
 
-our $VERSION = '0.001_01';
+our $VERSION = '0.002_01';
 
 use constant ARRAY_REF	=> ref [];
 
@@ -48,12 +48,18 @@ sub new {
     bless $self, ref $class || $class;
     $self->set_rules( $breed, $live );
 
+    $self->clear();
+
     return $self;
 }
 
 sub clear {
     my ( $self ) = @_;
     delete $self->{grid};
+    delete $self->{changed};
+    $self->{living_x} = [];
+    $self->{living_y} = [];
+    $self->{change_count} = 0;
     return $self;
 }
 
@@ -67,11 +73,11 @@ sub get_grid {
     if ( $self->{grid} ) {
 	my @rslt;
 	foreach my $x ( 0 .. $self->{max_x} ) {
-	    if ( $self->{grid}[$x] ) {
+	    if ( $self->{grid}{$x} ) {
 		push @rslt, [];
 		foreach my $y ( 0 .. $self->{max_y} ) {
-		    push @{ $rslt[-1] }, $self->{grid}[$x][$y] ?
-			$self->{grid}[$x][$y][0] ? 1 : 0 : 0;
+		    push @{ $rslt[-1] }, $self->{grid}{$x}{$y} ?
+			$self->{grid}{$x}{$y}[0] ? 1 : 0 : 0;
 		}
 	    } else {
 		push @rslt, [ ( 0 ) x $self->{size_y} ];
@@ -95,9 +101,9 @@ sub get_text_grid {
     my @rslt;
     if ( $self->{grid} ) {
 	foreach my $x ( 0 .. $self->{max_x} ) {
-	    if ( $self->{grid}[$x] ) {
+	    if ( $self->{grid}{$x} ) {
 		push @rslt, join '', map {
-		    ( $self->{grid}[$x][$_] && $self->{grid}[$x][$_][0]) ?
+		    ( $self->{grid}{$x}{$_} && $self->{grid}{$x}{$_}[0]) ?
 		    $living : $dead
 		} 0 .. $self->{max_y};
 	    } else {
@@ -110,6 +116,76 @@ sub get_text_grid {
     return wantarray ? @rslt : join '', map { "$_\n" } @rslt;
 }
 
+sub get_active_text_grid {
+    my ( $self, $living, $dead ) = @_;
+    $self->{grid}
+	or return;
+    $living	||= 'X';
+    $dead	||= '.';
+    my ( $min_x, $max_x, $min_y, $max_y ) = ( $self->{size_x}, 0,
+	$self->{size_y}, 0 );
+    foreach my $ix ( keys %{ $self->{changed} } ) {
+	$min_x = $ix if $ix < $min_x;
+	$max_x = $ix if $ix > $max_x;
+	foreach my $iy ( keys %{ $self->{changed}{$ix} } ) {
+	    $min_y = $iy if $iy < $min_y;
+	    $max_y = $iy if $iy > $max_y;
+	}
+    }
+    $max_x < $min_x
+	and return;
+    my $rslt;
+    foreach my $x ( $min_x .. $max_x ) {
+	foreach my $y ( $min_y .. $max_y ) {
+	    # Yes, I'm paranoid about autovivification.
+	    $rslt .= ( $self->{grid}{$x} && $self->{grid}{$x}{$y} &&
+		$self->{grid}{$x}{$y}[0] ) ? $living : $dead;
+	}
+	$rslt .= "\n";
+    }
+
+    return ( $min_x, $min_y, $rslt );
+}
+
+sub get_used_text_grid {
+    my ( $self, $living, $dead ) = @_;
+    $self->{grid}
+	or return;
+    $living	||= 'X';
+    $dead	||= '.';
+    my ( $min_x, $max_x, $min_y, $max_y );
+    for ( $min_x = 0; $min_x < $self->{size_x}; $min_x++ ) {
+	$self->{living_x}[$min_x]
+	    and last;
+    }
+    for ( $max_x = $self->{size_x}; $max_x >= $min_x; ) {
+	$self->{living_x}[--$max_x]
+	    and last;
+    }
+    $max_x < $min_x
+	and return;
+    for ( $min_y = 0; $min_y < $self->{size_y}; $min_y++ ) {
+	$self->{living_y}[$min_y]
+	    and last;
+    }
+    for ( $max_y = $self->{size_y}; $max_y >= $min_y; ) {
+	$self->{living_y}[--$max_y]
+	    and last;
+    }
+
+    my $rslt;
+    foreach my $x ( $min_x .. $max_x ) {
+	foreach my $y ( $min_y .. $max_y ) {
+	    # Yes, I'm paranoid about autovivification.
+	    $rslt .= ( $self->{grid}{$x} && $self->{grid}{$x}{$y} &&
+		$self->{grid}{$x}{$y}[0] ) ? $living : $dead;
+	}
+	$rslt .= "\n";
+    }
+
+    return ( $min_x, $min_y, $rslt );
+}
+
 sub get_used_grid {
     my ( $self ) = @_;
     $self->{grid}
@@ -117,16 +193,16 @@ sub get_used_grid {
     my @rslt;
     my $skip_x = 0;
     foreach my $x ( 0 .. $self->{max_x} ) {
-	if ( $self->{grid}[$x] ) {
+	if ( $self->{grid}{$x} ) {
 	    push @rslt, ( undef ) x $skip_x;
 	    my $skip_y = $skip_x = 0;
 	    foreach my $y ( 0 .. $self->{max_y} ) {
-		if ( $self->{grid}[$x][$y] &&
-		    defined $self->{grid}[$x][$y][0] ) {
+		if ( $self->{grid}{$x}{$y} &&
+		    defined $self->{grid}{$x}{$y}[0] ) {
 		    @rslt > $x
 			or push @rslt, [];
 		    push @{ $rslt[-1] }, ( undef ) x $skip_y,
-			$self->{grid}[$x][$y][0];
+			$self->{grid}{$x}{$y}[0];
 		    $skip_y = 0;
 		} else {
 		    $skip_y++;
@@ -174,32 +250,29 @@ sub process {
     my ( $self, $steps ) = @_;
     $steps ||= 1;
 
-    my @toggle;
-
     foreach ( 1 .. $steps ) {
-	@toggle = ();
 
-	foreach my $x ( keys %{ $self->{changed} } ) {
-	    foreach my $y ( keys %{ $self->{changed}{$x} } ) {
-		my $cell = $self->{grid}[$x][$y];
+	my $changed = delete $self->{changed};
+	$self->{change_count} = 0;
+
+	foreach my $x ( keys %{ $changed } ) {
+	    foreach my $y ( keys %{ $changed->{$x} } ) {
+		my $cell = $self->{grid}{$x}{$y};
 		no warnings qw{ uninitialized };
 		if ( $cell->[0] ) {
-		    $self->{live}[ $cell->[1] ]
-			or push @toggle, [ $x, $y, 0 ];
+		    $self->{live}[ $changed->{$x}{$y} ]
+			or $self->unset_point( $x, $y );
 		} else {
-		    $self->{breed}[ $cell->[1] ]
-			and push @toggle, [ $x, $y, 1 ];
+		    $self->{breed}[ $changed->{$x}{$y} ]
+			and $self->set_point( $x, $y );
 		}
 	    }
 	}
 
-	delete $self->{changed};
-
-	foreach my $cell ( @toggle ) {
-	    $self->set_point_state( @{ $cell } );
-	}
+	$self->{change_count}
+	    or last;
     }
-    return scalar @toggle;
+    return $self->{change_count};
 }
 
 sub set_point {
@@ -209,6 +282,7 @@ sub set_point {
 
 sub set_point_state {
     my ( $self, $x, $y, $state ) = @_;
+
     defined $x
 	and defined $y
 	and $x =~ NON_NEGATIVE_INTEGER_RE
@@ -216,39 +290,44 @@ sub set_point_state {
 	or croak 'Coordinates must be non-negative integers';
     defined $state
 	or return $state;
-    my $off_grid = ( $x < 0 || $x > $self->{max_x} ||
-	$y < 0 || $y > $self->{max_y} )
-	and $state
-	and croak 'Attempt to place living cell outside grid';
-    if ( TOGGLE_STATE_REF eq ref $state ) {
-	$state = 1;
-	$self->{grid}
-	    and $self->{grid}[$x]
-	    and $self->{grid}[$x][$y]
-	    and $state = ( ! $self->{grid}[$x][0] );
-    }
-    $state = $state ? 1 : 0;
-    my $prev_val = $off_grid ? 0 : $self->{grid}[$x][$y][0] ? 1 : 0;
-    unless ( $off_grid ) {
-	$self->{grid}[$x][$y][0] = $state;
-	$self->{grid}[$x][$y][1] ||= 0;
-    }
-    my $delta = $state - $prev_val
-	or return $state;
-    foreach my $ix ( max( 0, $x - 1 ) .. min( $self->{max_x}, $x + 1 ) ) {
-	foreach my $iy ( max( 0, $y - 1 ) .. min( $self->{max_y}, $y + 1 ) ) {
-	    $self->{grid}[$ix][$iy][1] += $delta;
-	    $self->{changed}{$ix}{$iy}++;
+
+    if ( $x >= 0 && $x < $self->{size_x} &&
+	$y >= 0 && $y < $self->{size_y}
+    ) {
+	# We're on-grid.
+
+	# This autovivifies, but we're going to assign it anyway, so ...
+	my $prev_state = $self->{grid}{$x}{$y}[0] || 0;
+	$state = TOGGLE_STATE_REF eq ref $state ? 1 - $prev_state :
+	    $state ? 1 : 0;
+
+	$self->{grid}{$x}{$y}[0] = $state;
+	$self->{grid}{$x}{$y}[1] ||= 0;
+	my $delta = $state - $prev_state
+	    or return $state;
+	$self->{living_x}[$x] += $delta;
+	$self->{living_y}[$y] += $delta;
+
+	$self->{change_count}++;
+
+	foreach my $ix ( max( 0, $x - 1 ) .. min( $self->{max_x}, $x + 1 ) ) {
+	    foreach my $iy ( max( 0, $y - 1 ) .. min( $self->{max_y}, $y + 1 )
+	    ) {
+		$self->{changed}{$ix}{$iy} =
+		    $self->{grid}{$ix}{$iy}[1] += $delta;
+	    }
 	}
+
+	# A cell is not its own neighbor, but the above nested loops
+	# assumed that it was. We fix that here, rather than skip it
+	# inside the loops.
+	$self->{changed}{$x}{$y} =
+	    $self->{grid}{$x}{$y}[1] -= $delta;
+
+    } elsif ( $state ) {
+	croak 'Attempt to place living cell outside grid';
     }
-    # A cell is not its own neighbor, but the above nested loops assumed
-    # that it was. We fix that here, rather than skip it inside the
-    # loops.
-    unless ( $off_grid ) {
-	$self->{grid}[$x][$y][1] -= $delta;
-	--$self->{changed}{$x}{$y}
-	    or delete $self->{changed}{$x}{$y};
-    }
+
     return $state;
 }
 
@@ -394,6 +473,24 @@ invocant.
 
 This method is an extension to L<Game::Life|Game::Life>.
 
+=head2 get_active_text_grid
+
+ my ( $x, $y, $grid ) = $life->get_active_text_grid()
+ print "${grid}at row $x column $y\n"
+
+This convenience method returns the active portion of the grid as
+text. Specifically, the returns are the number of the first row that
+contains an active cell, the number of the column that contains the first
+active cell, and the text grid with each line C<"\n">-terminated.
+
+A cell is considered active if its number of neighbors changed in the
+most-recent call to L<process()|/process>, as a result of any manual
+modifications since that call, or both.
+
+If there are no active cells, nothing is returned.
+
+If called in scalar context you get the active grid.
+
 =head2 get_breeding_rules
 
  use Data::Dumper;
@@ -473,6 +570,20 @@ As an incompatible change to the same-named method of
 L<Game::Life|Game::Life>, if called in scalar context this method
 returns a single string representing the entire grid.
 
+=head2 get_used_text_grid
+
+ my ( $x, $y, $grid ) = $life->get_used_text_grid()
+ print "${grid}at row $x column $y\n"
+
+This convenience method returns the living portion of the grid as
+text. Specifically, the returns are the number of the first row that
+contains a living cell, the number of the column that contains the first
+living cell, and the text grid with each line C<"\n">-terminated.
+
+If there are no living cells, nothing is returned.
+
+If called in scalar context you get the living portion of the grid.
+
 =head2 get_used_grid
 
  use Data::Dumper;
